@@ -1,21 +1,21 @@
-/* Copyright (C) 2009-2011, 2013-2015, 2018, 2019
- *   Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 2009-2011,2013-2015,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -25,18 +25,24 @@
 #include <string.h>
 #include <stdio.h>
 #include <assert.h>
-#include <intprops.h>
+#include "boolean.h"
+#include "bytevectors.h"
+#include "chars.h"
+#include "eval.h"
+#include "extensions.h"
+#include "gsubr.h"
+#include "numbers.h"
+#include "ports-internal.h"
+#include "procs.h"
+#include "smob.h"
+#include "strings.h"
+#include "symbols.h"
+#include "syscalls.h"
+#include "values.h"
+#include "vectors.h"
+#include "version.h"
 
-#include "libguile/_scm.h"
-#include "libguile/bytevectors.h"
-#include "libguile/chars.h"
-#include "libguile/eval.h"
-#include "libguile/r6rs-ports.h"
-#include "libguile/strings.h"
-#include "libguile/validate.h"
-#include "libguile/values.h"
-#include "libguile/vectors.h"
-#include "libguile/ports-internal.h"
+#include "r6rs-ports.h"
 
 
 
@@ -132,27 +138,21 @@ bytevector_input_port_seek (SCM port, scm_t_off offset, int whence)
 #define FUNC_NAME "bytevector_input_port_seek"
 {
   struct bytevector_input_port *stream = (void *) SCM_STREAM (port);
-  size_t base;
   scm_t_off target;
 
   if (whence == SEEK_CUR)
-    base = stream->pos;
+    target = offset + stream->pos;
   else if (whence == SEEK_SET)
-    base = 0;
+    target = offset;
   else if (whence == SEEK_END)
-    base = SCM_BYTEVECTOR_LENGTH (stream->bytevector);
+    target = offset + SCM_BYTEVECTOR_LENGTH (stream->bytevector);
   else
     scm_wrong_type_arg_msg (FUNC_NAME, 0, port, "invalid `seek' parameter");
 
-  if (base > SCM_T_OFF_MAX
-      || INT_ADD_OVERFLOW ((scm_t_off) base, offset))
-    scm_num_overflow (FUNC_NAME);
-  target = (scm_t_off) base + offset;
-
   if (target >= 0 && target <= SCM_BYTEVECTOR_LENGTH (stream->bytevector))
     stream->pos = target;
   else
-    scm_out_of_range (FUNC_NAME, scm_from_off_t (offset));
+    scm_out_of_range (FUNC_NAME, scm_from_long (offset));
 
   return target;
 }
@@ -232,9 +232,6 @@ custom_binary_port_seek (SCM port, scm_t_off offset, int whence)
 	  /* We just want to know the current position.  */
 	  break;
 
-        if (INT_ADD_OVERFLOW (offset, c_result))
-          scm_num_overflow (FUNC_NAME);
-
 	offset += c_result;
 	/* Fall through.  */
       }
@@ -242,7 +239,7 @@ custom_binary_port_seek (SCM port, scm_t_off offset, int whence)
     case SEEK_SET:
       {
 	if (SCM_LIKELY (scm_is_true (stream->set_position_x)))
-	  result = scm_call_1 (stream->set_position_x, scm_from_off_t (offset));
+	  result = scm_call_1 (stream->set_position_x, scm_from_int (offset));
 	else
 	  scm_wrong_type_arg_msg (FUNC_NAME, 0, port,
 				  "seekable R6RS custom binary port");
@@ -404,7 +401,7 @@ SCM_DEFINE (scm_lookahead_u8, "lookahead-u8", 1, 0, 0,
   if (u8 == EOF)
     result = SCM_EOF_VAL;
   else
-    result = SCM_I_MAKINUM ((scm_t_uint8) u8);
+    result = SCM_I_MAKINUM ((uint8_t) u8);
 
   return result;
 }
@@ -465,8 +462,7 @@ SCM_DEFINE (scm_get_bytevector_n_x, "get-bytevector-n!", 4, 0, 0,
 
   c_len = SCM_BYTEVECTOR_LENGTH (bv);
 
-  if (SCM_UNLIKELY (c_len < c_start
-                    || (c_len - c_start < c_count)))
+  if (SCM_UNLIKELY (c_start + c_count > c_len))
     scm_out_of_range (FUNC_NAME, count);
 
   if (SCM_LIKELY (c_count > 0))
@@ -507,56 +503,13 @@ SCM_DEFINE (scm_get_bytevector_some, "get-bytevector-some", 1, 0, 0,
     }
 
   bv = scm_c_make_bytevector (avail);
-  scm_port_buffer_take (buf, (scm_t_uint8 *) SCM_BYTEVECTOR_CONTENTS (bv),
+  scm_port_buffer_take (buf, (uint8_t *) SCM_BYTEVECTOR_CONTENTS (bv),
                         avail, cur, avail);
 
   return bv;
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_get_bytevector_some_x, "get-bytevector-some!", 4, 0, 0,
-	    (SCM port, SCM bv, SCM start, SCM count),
-            "Read up to @var{count} bytes from @var{port}, blocking "
-            "as necessary until at least one byte is available or an "
-            "end-of-file is reached.  Store them in @var{bv} starting "
-            "at index @var{start}.  Return the number of bytes actually "
-            "read, or an end-of-file object.")
-#define FUNC_NAME s_scm_get_bytevector_some_x
-{
-  SCM buf;
-  size_t c_start, c_count, c_len;
-  size_t cur, avail, transfer_size;
-
-  SCM_VALIDATE_BINARY_INPUT_PORT (1, port);
-  SCM_VALIDATE_BYTEVECTOR (2, bv);
-  c_start = scm_to_size_t (start);
-  c_count = scm_to_size_t (count);
-
-  c_len = SCM_BYTEVECTOR_LENGTH (bv);
-
-  if (SCM_UNLIKELY (c_len < c_start
-                    || c_len - c_start < c_count))
-    scm_out_of_range (FUNC_NAME, count);
-
-  if (c_count == 0)
-    return SCM_INUM0;
-
-  buf = scm_fill_input (port, 0, &cur, &avail);
-  if (avail == 0)
-    {
-      scm_port_buffer_set_has_eof_p (buf, SCM_BOOL_F);
-      return SCM_EOF_VAL;
-    }
-
-  transfer_size = min (avail, c_count);
-  scm_port_buffer_take (buf,
-                        (scm_t_uint8 *) SCM_BYTEVECTOR_CONTENTS (bv) + c_start,
-                        transfer_size, cur, avail);
-
-  return scm_from_size_t (transfer_size);
-}
-#undef FUNC_NAME
-
 SCM_DEFINE (scm_get_bytevector_all, "get-bytevector-all", 1, 0, 0,
 	    (SCM port),
 	    "Read from @var{port}, blocking as necessary, until "
@@ -566,7 +519,7 @@ SCM_DEFINE (scm_get_bytevector_all, "get-bytevector-all", 1, 0, 0,
 #define FUNC_NAME s_scm_get_bytevector_all
 {
   SCM result;
-  size_t c_len, c_count;
+  unsigned c_len, c_count;
   size_t c_read, c_total;
 
   SCM_VALIDATE_BINARY_INPUT_PORT (1, port);
@@ -577,14 +530,10 @@ SCM_DEFINE (scm_get_bytevector_all, "get-bytevector-all", 1, 0, 0,
 
   do
     {
-      if (c_read > c_len - c_total)
+      if (c_total + c_read > c_len)
 	{
 	  /* Grow the bytevector.  */
           SCM prev = result;
-
-          if (INT_ADD_OVERFLOW (c_len, c_len))
-            scm_num_overflow (FUNC_NAME);
-
           result = scm_c_make_bytevector (c_len * 2);
           memcpy (SCM_BYTEVECTOR_CONTENTS (result),
                   SCM_BYTEVECTOR_CONTENTS (prev),
@@ -624,7 +573,7 @@ SCM_DEFINE (scm_put_u8, "put-u8", 2, 0, 0,
 	    "Write @var{octet} to binary port @var{port}.")
 #define FUNC_NAME s_scm_put_u8
 {
-  scm_t_uint8 c_octet;
+  uint8_t c_octet;
 
   SCM_VALIDATE_BINARY_OUTPUT_PORT (1, port);
   c_octet = scm_to_uint8 (octet);
@@ -652,17 +601,20 @@ SCM_DEFINE (scm_put_bytevector, "put-bytevector", 2, 2, 0,
   if (!scm_is_eq (start, SCM_UNDEFINED))
     {
       c_start = scm_to_size_t (start);
-      if (SCM_UNLIKELY (c_start > c_len))
-        scm_out_of_range (FUNC_NAME, start);
 
       if (!scm_is_eq (count, SCM_UNDEFINED))
 	{
 	  c_count = scm_to_size_t (count);
-	  if (SCM_UNLIKELY (c_count > c_len - c_start))
+	  if (SCM_UNLIKELY (c_start + c_count > c_len))
 	    scm_out_of_range (FUNC_NAME, count);
 	}
       else
-        c_count = c_len - c_start;
+	{
+	  if (SCM_UNLIKELY (c_start > c_len))
+	    scm_out_of_range (FUNC_NAME, start);
+	  else
+	    c_count = c_len - c_start;
+	}
     }
   else
     c_start = 0, c_count = c_len;
@@ -692,17 +644,20 @@ SCM_DEFINE (scm_unget_bytevector, "unget-bytevector", 2, 2, 0,
   if (!scm_is_eq (start, SCM_UNDEFINED))
     {
       c_start = scm_to_size_t (start);
-      if (SCM_UNLIKELY (c_start > c_len))
-        scm_out_of_range (FUNC_NAME, start);
 
       if (!scm_is_eq (count, SCM_UNDEFINED))
 	{
 	  c_count = scm_to_size_t (count);
-	  if (SCM_UNLIKELY (c_count > c_len - c_start))
+	  if (SCM_UNLIKELY (c_start + c_count > c_len))
 	    scm_out_of_range (FUNC_NAME, count);
 	}
       else
-        c_count = c_len - c_start;
+	{
+	  if (SCM_UNLIKELY (c_start > c_len))
+	    scm_out_of_range (FUNC_NAME, start);
+	  else
+	    c_count = c_len - c_start;
+	}
     }
   else
     c_start = 0, c_count = c_len;
@@ -775,20 +730,17 @@ bytevector_output_port_buffer_grow (scm_t_bytevector_output_port_buffer *buf,
   char *new_buf;
   size_t new_size;
 
+  for (new_size = buf->total_len
+	 ? buf->total_len : SCM_BYTEVECTOR_OUTPUT_PORT_BUFFER_INITIAL_SIZE;
+       new_size < min_size;
+       new_size *= 2);
+
   if (buf->buffer)
-    {
-      if (INT_ADD_OVERFLOW (buf->total_len, buf->total_len))
-        scm_num_overflow ("bytevector_output_port_buffer_grow");
-      new_size = max (min_size, buf->total_len * 2);
-      new_buf = scm_gc_realloc ((void *) buf->buffer, buf->total_len,
-                                new_size, SCM_GC_BYTEVECTOR_OUTPUT_PORT);
-    }
+    new_buf = scm_gc_realloc ((void *) buf->buffer, buf->total_len,
+			      new_size, SCM_GC_BYTEVECTOR_OUTPUT_PORT);
   else
-    {
-      new_size = max (min_size, SCM_BYTEVECTOR_OUTPUT_PORT_BUFFER_INITIAL_SIZE);
-      new_buf = scm_gc_malloc_pointerless (new_size,
-                                           SCM_GC_BYTEVECTOR_OUTPUT_PORT);
-    }
+    new_buf = scm_gc_malloc_pointerless (new_size,
+                                         SCM_GC_BYTEVECTOR_OUTPUT_PORT);
 
   buf->buffer = new_buf;
   buf->total_len = new_size;
@@ -813,24 +765,19 @@ make_bytevector_output_port (void)
 
   SCM_NEWSMOB (proc, bytevector_output_port_procedure, buf);
 
-  return scm_values (scm_list_2 (port, proc));
+  return scm_values_2 (port, proc);
 }
 
 /* Write octets from WRITE_BUF to the backing store.  */
 static size_t
 bytevector_output_port_write (SCM port, SCM src, size_t start, size_t count)
-#define FUNC_NAME "bytevector_output_port_write"
 {
   scm_t_bytevector_output_port_buffer *buf;
 
   buf = SCM_BYTEVECTOR_OUTPUT_PORT_BUFFER (port);
 
-  if (count > buf->total_len - buf->pos)
-    {
-      if (INT_ADD_OVERFLOW (buf->pos, count))
-        scm_num_overflow (FUNC_NAME);
-      bytevector_output_port_buffer_grow (buf, buf->pos + count);
-    }
+  if (buf->pos + count > buf->total_len)
+    bytevector_output_port_buffer_grow (buf, buf->pos + count);
 
   memcpy (buf->buffer + buf->pos, SCM_BYTEVECTOR_CONTENTS (src) + start, count);
 
@@ -839,36 +786,29 @@ bytevector_output_port_write (SCM port, SCM src, size_t start, size_t count)
 
   return count;
 }
-#undef FUNC_NAME
 
 static scm_t_off
 bytevector_output_port_seek (SCM port, scm_t_off offset, int whence)
 #define FUNC_NAME "bytevector_output_port_seek"
 {
   scm_t_bytevector_output_port_buffer *buf;
-  size_t base;
   scm_t_off target;
 
   buf = SCM_BYTEVECTOR_OUTPUT_PORT_BUFFER (port);
 
   if (whence == SEEK_CUR)
-    base = buf->pos;
+    target = offset + buf->pos;
   else if (whence == SEEK_SET)
-    base = 0;
+    target = offset;
   else if (whence == SEEK_END)
-    base = buf->len;
+    target = offset + buf->len;
   else
     scm_wrong_type_arg_msg (FUNC_NAME, 0, port, "invalid `seek' parameter");
 
-  if (base > SCM_T_OFF_MAX
-      || INT_ADD_OVERFLOW ((scm_t_off) base, offset))
-    scm_num_overflow (FUNC_NAME);
-  target = (scm_t_off) base + offset;
-
   if (target >= 0 && target <= buf->len)
     buf->pos = target;
   else
-    scm_out_of_range (FUNC_NAME, scm_from_off_t (offset));
+    scm_out_of_range (FUNC_NAME, scm_from_long (offset));
 
   return target;
 }
@@ -1238,5 +1178,5 @@ scm_register_r6rs_ports (void)
 void
 scm_init_r6rs_ports (void)
 {
-#include "libguile/r6rs-ports.x"
+#include "r6rs-ports.x"
 }
